from cone.app.browser.ajax import AjaxAction
from cone.app.browser.ajax import ajax_continue
from cone.app.browser.ajax import ajax_message
from cone.app.browser.form import Form
from cone.app.browser.form import YAMLForm
from cone.app.browser.layout import ProtectedContentTile
from cone.app.browser.settings import SettingsBehavior
from cone.app.browser.utils import make_url
from cone.app.ugm import ugm_backend
from cone.ldap.settings import LDAPGroupsSettings
from cone.ldap.settings import LDAPRolesSettings
from cone.ldap.settings import LDAPServerSettings
from cone.ldap.settings import LDAPUsersSettings
from cone.tile import Tile
from cone.tile import tile
from cone.ugm.utils import general_settings
from node.ext.ldap import BASE
from node.ext.ldap import ONELEVEL
from node.ext.ldap import SUBTREE
from node.utils import UNSET
from odict import odict
from plumber import plumbing
from pyramid.i18n import TranslationStringFactory
from pyramid.i18n import get_localizer
from yafowil.base import ExtractionError


_ = TranslationStringFactory('cone.ldap')


def initialize_ugm_backend():
    if ugm_backend.name == 'ldap':
        ugm_backend.initialize()


class ScopeVocabMixin(object):
    scope_vocab = [
        (str(BASE), 'BASE'),
        (str(ONELEVEL), 'ONELEVEL'),
        (str(SUBTREE), 'SUBTREE'),
    ]


class CreateContainerTrigger(Tile):

    @property
    def creation_target(self):
        return make_url(self.request, node=self.model)

    @property
    def ldap_connectivity(self):
        return self.model.parent['ldap_server'].ldap_connectivity


class CreateContainerAction(Tile):

    @property
    def continuation(self):
        raise NotImplementedError(
            'Abstract ``CreateContainerAction`` '
            'does not implement ``continuation``'
        )

    def render(self):
        try:
            message = self.model.create_container()
            ajax_message(self.request, message, 'info')
            continuation = self.continuation
            ajax_continue(self.request, continuation)
        except Exception as e:
            localizer = get_localizer(self.request)
            message = localizer.translate(_(
                'cannot_create_container',
                default="Cannot create container: ${error}",
                mapping={'error': localizer.translate(e.error_message)}
            ))
            ajax_message(self.request, message, 'error')
        # XXX: initialize UGM backend?
        return u''


class AliasesMixin(object):
    reserved_alias_names = []
    aliases_attribute = ''

    @property
    def reserved_aliases_value(self):
        attrs = self.model.attrs
        value = getattr(attrs, self.aliases_attribute)
        reserved_aliases = odict()
        for attr_name in self.reserved_alias_names:
            reserved_aliases[attr_name] = value.get(attr_name)
        return reserved_aliases

    @property
    def additional_aliases_value(self):
        attrs = self.model.attrs
        value = getattr(attrs, self.aliases_attribute)
        additional_aliases = odict()
        for attr_name in value.keys():
            if attr_name in self.reserved_alias_names:
                continue
            additional_aliases[attr_name] = value.get(attr_name)
        return additional_aliases

    def additional_aliases_extractor(self, widget, data):
        extracted = data.extracted
        if extracted is UNSET:
            return extracted
        for val in extracted.values():
            if not val.strip():
                raise ExtractionError(_(
                    'additional_aliases_empty_string',
                    default='Additional aliases must not be empty'
                ))
        return extracted


@tile(
    name='content',
    path='templates/server_settings.pt',
    interface=LDAPServerSettings,
    permission='manage')
class ServerSettingsTile(ProtectedContentTile):

    @property
    def ldap_status(self):
        if self.model.ldap_connectivity:
            return 'OK'
        return _('server_down', default='Down')


@tile(name='editform', interface=LDAPServerSettings, permission='manage')
@plumbing(SettingsBehavior, YAMLForm)
class ServerSettingsForm(Form):
    action_resource = u'edit'
    form_template = 'cone.ldap.browser:forms/server_settings.yaml'

    @property
    def message_factory(self):
        return _

    def save(self, widget, data):
        model = self.model
        for attr_name in ['uri', 'user']:
            val = data.fetch('ldap_server_settings.%s' % attr_name).extracted
            setattr(model.attrs, attr_name, val)
        cache = data.fetch('ldap_server_settings.cache').extracted
        cache = str(int(cache))
        setattr(model.attrs, 'cache', cache)
        password = data.fetch('ldap_server_settings.password').extracted
        if password is not UNSET:
            setattr(model.attrs, 'password', password)
        model()
        model.invalidate()
        initialize_ugm_backend()


@tile(
    name='content',
    path='templates/users_settings.pt',
    interface=LDAPUsersSettings,
    permission='manage')
class UsersSettingsTile(ProtectedContentTile, CreateContainerTrigger):

    @property
    def ldap_users(self):
        if self.model.container_exists:
            return 'OK'
        return _('inexistent', default='Inexistent')


@tile(name='create_container', interface=LDAPUsersSettings, permission='manage')
class UsersCreateContainerAction(CreateContainerAction):

    @property
    def continuation(self):
        url = make_url(self.request, node=self.model)
        return AjaxAction(url, 'content', 'inner', '.ldap_users')


@tile(name='editform', interface=LDAPUsersSettings, permission='manage')
@plumbing(SettingsBehavior, YAMLForm)
class UsersSettingsForm(Form, ScopeVocabMixin, AliasesMixin):
    action_resource = u'edit'
    form_template = 'cone.ldap.browser:forms/users_settings.yaml'
    reserved_alias_names = ['rdn', 'id', 'password']
    aliases_attribute = 'users_aliases_attrmap'

    @property
    def message_factory(self):
        return _

    def required_if_users_account_expiration(self, widget, data):
        extracted = data.extracted
        if extracted is UNSET:
            return extracted
        settings = general_settings(self.model)
        if settings.attrs.users_account_expiration == 'True' and not extracted:
            raise ExtractionError(_(
                'required_if_users_account_expiration',
                default='Value is required if account expiration is enabled'
            ))
        return extracted

    def save(self, widget, data):
        model = self.model
        for attr_name in [
            'users_dn',
            'users_scope',
            'users_query',
            'users_object_classes',
            'users_expires_attr',
            'users_expires_unit'
        ]:
            val = data.fetch('ldap_users_settings.%s' % attr_name).extracted
            if attr_name == 'users_object_classes':
                val = [v.strip() for v in val.split(',') if v.strip()]
            setattr(model.attrs, attr_name, val)
        attr_map = data.fetch('ldap_users_settings.users_additional_aliases').extracted
        attr_map.update(data.fetch('ldap_users_settings.users_reserved_aliases').extracted)
        model.attrs.users_aliases_attrmap = attr_map
        model()
        model.invalidate()
        initialize_ugm_backend()


@tile(
    name='content',
    path='templates/groups_settings.pt',
    interface=LDAPGroupsSettings,
    permission='manage')
class GroupsSettingsTile(ProtectedContentTile, CreateContainerTrigger):

    @property
    def ldap_groups(self):
        if self.model.container_exists:
            return 'OK'
        return _('inexistent', default='Inexistent')


@tile(name='create_container', interface=LDAPGroupsSettings, permission='manage')
class GroupsCreateContainerAction(CreateContainerAction):

    @property
    def continuation(self):
        url = make_url(self.request, node=self.model)
        return AjaxAction(url, 'content', 'inner', '.ldap_groups')


@tile(name='editform', interface=LDAPGroupsSettings, permission='manage')
@plumbing(SettingsBehavior, YAMLForm)
class GroupsSettingsForm(Form, ScopeVocabMixin, AliasesMixin):
    action_resource = u'edit'
    form_template = 'cone.ldap.browser:forms/groups_settings.yaml'
    reserved_alias_names = ['rdn', 'id']
    aliases_attribute = 'groups_aliases_attrmap'

    @property
    def message_factory(self):
        return _

    def save(self, widget, data):
        model = self.model
        for attr_name in [
            'groups_dn',
            'groups_scope',
            'groups_query',
            'groups_object_classes'
            # 'groups_relation'
        ]:
            val = data.fetch('ldap_groups_settings.%s' % attr_name).extracted
            if attr_name == 'groups_object_classes':
                val = [v.strip() for v in val.split(',') if v.strip()]
            setattr(model.attrs, attr_name, val)
        attr_map = data.fetch('ldap_groups_settings.groups_additional_aliases').extracted
        attr_map.update(data.fetch('ldap_groups_settings.groups_reserved_aliases').extracted)
        model.attrs.groups_aliases_attrmap = attr_map
        model()
        model.invalidate()
        initialize_ugm_backend()


@tile(
    name='content',
    path='templates/roles_settings.pt',
    interface=LDAPRolesSettings,
    permission='manage')
class RolesSettingsTile(ProtectedContentTile, CreateContainerTrigger):

    @property
    def ldap_roles(self):
        if self.model.container_exists:
            return 'OK'
        return _('inexistent', default='Inexistent')


@tile(name='create_container', interface=LDAPRolesSettings, permission='manage')
class RolesCreateContainerAction(CreateContainerAction):

    @property
    def continuation(self):
        url = make_url(self.request, node=self.model)
        return AjaxAction(url, 'content', 'inner', '.ldap_roles')


@tile(name='editform', interface=LDAPRolesSettings, permission='manage')
@plumbing(SettingsBehavior, YAMLForm)
class RolesSettingsForm(Form, ScopeVocabMixin):
    action_resource = u'edit'
    form_template = 'cone.ldap.browser:forms/roles_settings.yaml'

    @property
    def message_factory(self):
        return _

    @property
    def roles_aliases_attrmap(self):
        attrs = self.model.attrs
        roles_aliases_attrmap = odict()
        for attr_name in ['rdn', 'id']:
            value = attrs.roles_aliases_attrmap.get(attr_name)
            roles_aliases_attrmap[attr_name] = value
        return roles_aliases_attrmap

    def save(self, widget, data):
        model = self.model
        for attr_name in [
            'roles_dn',
            'roles_scope',
            'roles_query',
            'roles_object_classes',
            'roles_aliases_attrmap'
            # 'roles_relation',
        ]:
            val = data.fetch('ldap_roles_settings.%s' % attr_name).extracted
            if attr_name == 'roles_object_classes':
                val = [v.strip() for v in val.split(',') if v.strip()]
            setattr(model.attrs, attr_name, val)
        model()
        model.invalidate()
        initialize_ugm_backend()



from cone.app import get_root
from cone.ldap import testing
from cone.ldap.settings import LDAPContainerError
from cone.ldap.settings import LDAPContainerSettings
from cone.ldap.settings import LDAPGroupsSettings
from cone.ldap.settings import LDAPRolesSettings
from cone.ldap.settings import LDAPServerSettings
from cone.ldap.settings import LDAPUsersSettings
from node.ext.ldap import LDAPProps
from node.ext.ldap.ugm import GroupsConfig
from node.ext.ldap.ugm import RolesConfig
from node.ext.ldap.ugm import UsersConfig
from node.tests import NodeTestCase


class TestSettings(NodeTestCase):
    layer = testing.ldap_layer

    @testing.invalidate_settings
    def test_LDAPServerSettings(self):
        settings = get_root()['settings']['ldap_server']

        self.assertTrue(isinstance(settings, LDAPServerSettings))

        md = settings.metadata
        self.assertEqual(md.title, 'server_settings_node')
        self.assertEqual(md.description, 'server_settings_node_description')

        attrs = settings.attrs
        self.assertEqual(sorted(attrs.keys()), [
            'cache',
            'password',
            'uri',
            'user'
        ])

        attrs.uri = 'ldap://127.0.0.1:12345'
        attrs.user = 'cn=Manager,dc=my-domain,dc=com'
        attrs.password = 'secret'
        attrs.cache = '0'

        ldap_props = settings.ldap_props
        self.assertEqual(isinstance(ldap_props, LDAPProps), True)
        self.assertEqual(ldap_props.uri, attrs.uri)
        self.assertEqual(ldap_props.user, attrs.user)
        self.assertEqual(ldap_props.password, attrs.password)
        self.assertEqual(ldap_props.cache, int(attrs.cache))

        self.assertTrue(settings.attrs is attrs)
        self.assertTrue(settings.ldap_props is ldap_props)
        settings.invalidate()
        self.assertFalse(settings.attrs is attrs)
        self.assertFalse(settings.ldap_props is ldap_props)

        settings.invalidate()
        self.assertTrue(settings.ldap_connectivity)

        settings.invalidate()
        settings.attrs.uri = ''
        self.assertFalse(settings.ldap_connectivity)

        settings.invalidate()
        settings.attrs.cache = None
        self.assertFalse(settings.ldap_connectivity)

    def test_LDAPContainerSettings(self):
        settings = LDAPContainerSettings()
        settings.__parent__ = get_root()['settings']

        self.assertTrue(isinstance(
            settings.server_settings,
            LDAPServerSettings
        ))

        settings.container_dn = None
        err = self.expectError(LDAPContainerError, settings.create_container)
        self.assertEqual(err.error_message, 'no_container_dn_defined')
        self.assertFalse(settings.container_exists)

        settings.container_dn = 'uid=foo,dc=my-domain,dc=com'
        err = self.expectError(LDAPContainerError, settings.create_container)
        self.assertEqual(err.error_message, 'expected_ou_as_rdn')
        self.assertFalse(settings.container_exists)

        settings.container_dn = 'ou=container,#'
        err = self.expectError(LDAPContainerError, settings.create_container)
        self.assertEqual(err.error_message, 'invalid_dn')
        self.assertFalse(settings.container_exists)

        settings.container_dn = 'ou=container,ou=inexistent,dc=my-domain,dc=com'
        err = self.expectError(LDAPContainerError, settings.create_container)
        self.assertEqual(err.error_message, 'parent_not_found')
        self.assertFalse(settings.container_exists)

        settings.container_dn = 'ou=container,dc=my-domain,dc=com'
        settings.create_container()
        self.assertTrue(settings.container_exists)

    @testing.invalidate_settings
    def test_LDAPUsersSettings(self):
        settings = get_root()['settings']['ldap_users']

        self.assertTrue(isinstance(settings, LDAPUsersSettings))

        md = settings.metadata
        self.assertEqual(md.title, 'user_settings_node')
        self.assertEqual(md.description, 'user_settings_node_description')

        attrs = settings.attrs
        self.assertEqual(sorted(attrs.keys()), [
            'users_aliases_attrmap',
            'users_dn',
            'users_expires_attr',
            'users_expires_unit',
            'users_object_classes',
            'users_query',
            'users_scope'
        ])

        attrs.users_dn = 'ou=users,dc=my-domain,dc=com'
        attrs.users_scope = '1'
        attrs.users_query = ''
        attrs.users_object_classes = [
            'top',
            'person',
            'organizationalPerson',
            'inetOrgPerson'
        ]
        attrs.users_aliases_attrmap = {
            'rdn': 'uid',
            'id': 'uid',
            'password': 'userPassword'
        }

        ugm_settings = settings.parent['ugm_general']
        ugm_attrs = ugm_settings.attrs
        ugm_attrs.users_login_name_attr = 'uid'
        ugm_attrs.users_form_attrmap = {
            'cn': 'Fullname',
            'sn': 'Surname',
            'mail': 'Email'
        }
        ugm_attrs.users_exposed_attributes = ['homePhone']
        ugm_attrs.users_account_expiration = 'True'
        ugm_attrs.users_expires_attr = 'shadowExpire'
        ugm_attrs.users_expires_unit = '0'
        ugm_attrs.users_portrait = 'True'
        ugm_attrs.users_portrait_attr = 'jpegPhoto'

        ldap_ucfg = settings.ldap_ucfg
        self.assertEqual(isinstance(ldap_ucfg, UsersConfig), True)
        self.assertEqual(
            ldap_ucfg.baseDN,
            'ou=users,dc=my-domain,dc=com'
        )
        self.assertEqual(sorted(ldap_ucfg.attrmap.items()), [
            ('cn', 'cn'),
            ('homePhone', 'homePhone'),
            ('id', 'uid'),
            ('jpegPhoto', 'jpegPhoto'),
            ('login', 'uid'),
            ('mail', 'mail'),
            ('password', 'userPassword'),
            ('rdn', 'uid'),
            ('shadowExpire', 'shadowExpire'),
            ('sn', 'sn'),
            ('uid', 'uid')
        ])
        self.assertEqual(ldap_ucfg.scope, 1)
        self.assertEqual(ldap_ucfg.queryFilter, '')
        self.assertEqual(sorted(ldap_ucfg.objectClasses), [
            'inetOrgPerson',
            'organizationalPerson',
            'person',
            'top'
        ])
        self.assertEqual(ldap_ucfg.defaults, {})
        self.assertEqual(ldap_ucfg.expiresAttr, 'shadowExpire')
        self.assertEqual(ldap_ucfg.expiresUnit, 0)

        self.assertTrue(settings.attrs is attrs)
        self.assertTrue(settings.ldap_ucfg is ldap_ucfg)
        settings.invalidate()
        self.assertFalse(settings.attrs is attrs)
        self.assertFalse(settings.ldap_ucfg is ldap_ucfg)

        ldap_settings = settings.parent['ldap_server']
        ldap_settings.invalidate()
        settings.invalidate()
        settings.attrs.users_dn = 'ou=users-container,dc=my-domain,dc=com'
        self.assertFalse(settings.container_exists)
        settings.create_container()
        self.assertTrue(settings.container_exists)

        ldap_settings.invalidate()
        settings.invalidate()
        ldap_settings.attrs.cache = None
        self.assertFalse(settings.container_exists)

    @testing.invalidate_settings
    def test_LDAPGroupsSettings(self):
        settings = get_root()['settings']['ldap_groups']

        self.assertTrue(isinstance(settings, LDAPGroupsSettings))

        md = settings.metadata
        self.assertEqual(md.title, 'group_settings_node')
        self.assertEqual(md.description, 'group_settings_node_description')

        attrs = settings.attrs
        self.assertEqual(sorted(attrs.keys()), [
            'groups_aliases_attrmap',
            'groups_dn',
            'groups_object_classes',
            'groups_query',
            'groups_relation',
            'groups_scope'
        ])

        attrs.groups_dn = 'ou=groups,dc=my-domain,dc=com'
        attrs.groups_scope = '1'
        attrs.groups_query = ''
        attrs.groups_object_classes = ['groupOfNames']
        attrs.groups_aliases_attrmap = {
            'rdn': 'cn',
            'id': 'cn'
        }
        attrs.groups_relation = ''

        ugm_settings = settings.parent['ugm_general']
        ugm_attrs = ugm_settings.attrs
        ugm_attrs.groups_form_attrmap = {
            'description': 'Description'
        }

        ldap_gcfg = settings.ldap_gcfg
        self.assertEqual(isinstance(ldap_gcfg, GroupsConfig), True)

        self.assertEqual(
            ldap_gcfg.baseDN,
            'ou=groups,dc=my-domain,dc=com'
        )
        self.assertEqual(sorted(ldap_gcfg.attrmap.items()), [
            ('description', 'description'),
            ('id', 'cn'),
            ('rdn', 'cn')
        ])
        self.assertEqual(ldap_gcfg.scope, 1)
        self.assertEqual(ldap_gcfg.queryFilter, '')
        self.assertEqual(ldap_gcfg.objectClasses, ['groupOfNames'])
        self.assertEqual(ldap_gcfg.defaults, {})

        self.assertTrue(settings.attrs is attrs)
        self.assertTrue(settings.ldap_gcfg is ldap_gcfg)
        settings.invalidate()
        self.assertFalse(settings.attrs is attrs)
        self.assertFalse(settings.ldap_gcfg is ldap_gcfg)

        ldap_settings = settings.parent['ldap_server']
        ldap_settings.invalidate()
        settings.invalidate()
        settings.attrs.groups_dn = 'ou=groups-container,dc=my-domain,dc=com'
        self.assertFalse(settings.container_exists)
        settings.create_container()
        self.assertTrue(settings.container_exists)

        ldap_settings.invalidate()
        settings.invalidate()
        ldap_settings.attrs.cache = None
        self.assertFalse(settings.container_exists)

    @testing.invalidate_settings
    def test_LDAPRolesSettings(self):
        settings = get_root()['settings']['ldap_roles']

        self.assertTrue(isinstance(settings, LDAPRolesSettings))

        md = settings.metadata
        self.assertEqual(md.title, 'role_settings_node')
        self.assertEqual(md.description, 'role_settings_node_description')

        attrs = settings.attrs
        self.assertEqual(sorted(attrs.keys()), [
            'roles_aliases_attrmap',
            'roles_dn',
            'roles_object_classes',
            'roles_query',
            'roles_relation',
            'roles_scope'
        ])

        attrs.roles_dn = 'ou=roles,dc=my-domain,dc=com'
        attrs.roles_scope = '1'
        attrs.roles_query = ''
        attrs.roles_object_classes = ['groupOfUniqueNames']
        attrs.roles_aliases_attrmap = {
            'rdn': 'cn',
            'id': 'cn'
        }
        attrs.roles_relation = ''

        ldap_rcfg = settings.ldap_rcfg
        self.assertEqual(isinstance(ldap_rcfg, RolesConfig), True)

        self.assertEqual(
            ldap_rcfg.baseDN,
            'ou=roles,dc=my-domain,dc=com'
        )
        self.assertEqual(sorted(ldap_rcfg.attrmap.items()), [
            ('id', 'cn'),
            ('rdn', 'cn')
        ])
        self.assertEqual(ldap_rcfg.scope, 1)
        self.assertEqual(ldap_rcfg.queryFilter, '')
        self.assertEqual(ldap_rcfg.objectClasses, ['groupOfUniqueNames'])
        self.assertEqual(ldap_rcfg.defaults, {})

        self.assertTrue(settings.attrs is attrs)
        self.assertTrue(settings.ldap_rcfg is ldap_rcfg)
        settings.invalidate()
        self.assertFalse(settings.attrs is attrs)
        self.assertFalse(settings.ldap_rcfg is ldap_rcfg)

        ldap_settings = settings.parent['ldap_server']
        ldap_settings.invalidate()
        settings.invalidate()
        settings.attrs.roles_dn = 'ou=roles-container,dc=my-domain,dc=com'
        self.assertFalse(settings.container_exists)
        settings.create_container()
        self.assertTrue(settings.container_exists)

        ldap_settings.invalidate()
        settings.invalidate()
        ldap_settings.attrs.cache = None
        self.assertFalse(settings.container_exists)



from cone.app.model import Metadata
from cone.app.model import Properties
from cone.app.utils import format_traceback
from cone.ugm.settings import UGMSettings
from cone.ugm.utils import general_settings
from ldap.functions import explode_dn
from node.ext.ldap import LDAPNode
from node.ext.ldap import LDAPProps
from node.ext.ldap import testLDAPConnectivity
from node.ext.ldap.ugm import GroupsConfig
from node.ext.ldap.ugm import RolesConfig
from node.ext.ldap.ugm import UsersConfig
from node.ext.ldap.ugm._api import EXPIRATION_DAYS
from node.utils import instance_property
from odict import odict
from pyramid.i18n import TranslationStringFactory
import logging


logger = logging.getLogger('cone.ldap')
_ = TranslationStringFactory('cone.ldap')


# settings config
ldap_cfg = Properties()

# gets set by main hook
ldap_cfg.server_config = ''
ldap_cfg.users_config = ''
ldap_cfg.groups_config = ''
ldap_cfg.roles_config = ''

# user and group factory defaults
factory_defaults = Properties()
factory_defaults.user = dict()
factory_defaults.group = dict()
factory_defaults.role = dict()


class LDAPServerSettings(UGMSettings):

    @property
    def config_file(self):
        return ldap_cfg.server_config

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _(
            'server_settings_node',
            default='LDAP Server'
        )
        metadata.description = _(
            'server_settings_node_description',
            default='General LDAP Server Settings'
        )
        return metadata

    @property
    def ldap_connectivity(self):
        try:
            props = self.ldap_props
        except Exception:
            logger.error(format_traceback())
            return False
        return testLDAPConnectivity(props=props) == 'success'

    @instance_property
    def ldap_props(self):
        settings = self.attrs
        return LDAPProps(
            uri=settings.uri,
            user=settings.user,
            password=settings.password,
            cache=int(settings.cache)
        )

    def invalidate(self):
        super(LDAPServerSettings, self).invalidate(attrs=['ldap_props'])


class LDAPContainerError(Exception):
    """Error thrown if creating a LDAP container fails.

    ``error_message`` containes a translation string.
    """

    def __init__(self, error_message):
        super(LDAPContainerError, self).__init__()
        self.error_message = error_message


class LDAPContainerSettings(UGMSettings):
    container_dn = None

    @property
    def server_settings(self):
        return self.root['settings']['ldap_server']

    @property
    def container_exists(self):
        try:
            return LDAPNode(
                self.container_dn,
                self.server_settings.ldap_props
            ).exists
        except Exception:
            logger.error(format_traceback())
            return False

    def create_container(self):
        """Create LDAP container by dn.

        Currently this only supports ou container type.

        XXX: Do we need to support c and dc?
        XXX: Should we create parents as well if missing?
        """
        dn = self.container_dn
        if not dn:
            raise LDAPContainerError(_(
                'no_container_dn_defined',
                default='No container DN defined.'
            ))
        if not dn.startswith('ou='):
            raise LDAPContainerError(_(
                'expected_ou_as_rdn',
                default="Expected 'ou' as RDN Attribute."
            ))
        props = self.server_settings.ldap_props
        try:
            parent_dn = ','.join(explode_dn(dn)[1:])
        except Exception:
            raise LDAPContainerError(_(
                'invalid_dn',
                default='Invalid DN.'
            ))
        rdn = explode_dn(dn)[0]
        parent = LDAPNode(parent_dn, props)
        if not parent.exists:
            raise LDAPContainerError(_(
                'parent_not_found',
                default="Parent not found. Can't continue."
            ))
        parent[rdn] = LDAPNode()
        parent[rdn].attrs['objectClass'] = ['organizationalUnit']
        parent()
        self.invalidate()
        message = _(
            'created_principal_container',
            default="Created ${rdn}",
            mapping={'rdn': rdn}
        )
        return message


class LDAPUsersSettings(LDAPContainerSettings):

    @property
    def config_file(self):
        return ldap_cfg.users_config

    @property
    def container_dn(self):
        return self.attrs.users_dn

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _(
            'user_settings_node',
            default='LDAP Users'
        )
        metadata.description = _(
            'user_settings_node_description',
            default='User specific LDAP Settings'
        )
        return metadata

    @instance_property
    def ldap_ucfg(self):
        ugm_settings = general_settings(self).attrs
        settings = self.attrs
        attr_map = odict(settings.users_aliases_attrmap.items())
        login_name = ugm_settings.users_login_name_attr
        if login_name:
            attr_map['login'] = login_name
            if login_name not in attr_map:
                attr_map[login_name] = login_name
        else:
            # XXX: Not sure whether login attr fallback is needed. Keep for now
            #      since this is the behavior as before introducing
            #      users_login_name_attr setting.
            attr_map['login'] = attr_map['id']
        for attr in ugm_settings.users_form_attrmap:
            if attr in attr_map:
                continue
            attr_map[attr] = attr
        if ugm_settings.users_exposed_attributes:
            for attr in ugm_settings.users_exposed_attributes:
                if attr in attr_map:
                    continue
                attr_map[attr] = attr
        expires_attr = None
        expires_unit = EXPIRATION_DAYS
        if ugm_settings.users_account_expiration == 'True':
            expires_attr = settings.users_expires_attr
            expires_unit = int(settings.users_expires_unit)
            if expires_attr not in attr_map:
                attr_map[expires_attr] = expires_attr
        if ugm_settings.users_portrait == 'True':
            image_attr = ugm_settings.users_portrait_attr
            if image_attr not in attr_map:
                attr_map[image_attr] = image_attr
        return UsersConfig(
            baseDN=settings.users_dn,
            attrmap=attr_map,
            scope=int(settings.users_scope),
            queryFilter=settings.users_query,
            objectClasses=settings.users_object_classes,
            defaults=factory_defaults.user,
            expiresAttr=expires_attr,
            expiresUnit=expires_unit
        )

    def invalidate(self):
        super(LDAPUsersSettings, self).invalidate(attrs=['ldap_ucfg'])


class LDAPGroupsSettings(LDAPContainerSettings):

    @property
    def config_file(self):
        return ldap_cfg.groups_config

    @property
    def container_dn(self):
        return self.attrs.groups_dn

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _(
            'group_settings_node',
            default='LDAP Groups')
        metadata.description = _(
            'group_settings_node_description',
            default='Group specific LDAP Settings'
        )
        return metadata

    @instance_property
    def ldap_gcfg(self):
        ugm_settings = general_settings(self).attrs
        settings = self.attrs
        attr_map = odict(settings.groups_aliases_attrmap.items())
        for attr in ugm_settings.groups_form_attrmap:
            if attr in attr_map:
                continue
            attr_map[attr] = attr
        return GroupsConfig(
            baseDN=settings.groups_dn,
            attrmap=attr_map,
            scope=int(settings.groups_scope),
            queryFilter=settings.groups_query,
            objectClasses=settings.groups_object_classes,
            # member_relation=settings.groups_relation,
            defaults=factory_defaults.group
        )

    def invalidate(self):
        super(LDAPGroupsSettings, self).invalidate(attrs=['ldap_gcfg'])


class LDAPRolesSettings(LDAPContainerSettings):

    @property
    def config_file(self):
        return ldap_cfg.roles_config

    @property
    def container_dn(self):
        return self.attrs.roles_dn

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _(
            'role_settings_node',
            default='LDAP Roles')
        metadata.description = _(
            'role_settings_node_description',
            default='Role specific LDAP Settings'
        )
        return metadata

    @instance_property
    def ldap_rcfg(self):
        settings = self.attrs
        attr_map = odict(settings.roles_aliases_attrmap.items())
        return RolesConfig(
            baseDN=settings.roles_dn,
            attrmap=attr_map,
            scope=int(settings.roles_scope),
            queryFilter=settings.roles_query,
            objectClasses=settings.roles_object_classes,
            # member_relation=settings.roles_relation,
            defaults=factory_defaults.role
        )

    def invalidate(self):
        super(LDAPRolesSettings, self).invalidate(attrs=['ldap_rcfg'])



import sys
import unittest


def test_suite():
    from cone.ldap.tests import test_ldap
    from cone.ldap.tests import test_settings
    from cone.ldap.tests import test_ugm

    from cone.ldap.tests import test_browser_settings

    suite = unittest.TestSuite()

    suite.addTest(unittest.findTestCases(test_ldap))
    suite.addTest(unittest.findTestCases(test_settings))
    suite.addTest(unittest.findTestCases(test_ugm))

    suite.addTest(unittest.findTestCases(test_browser_settings))

    return suite


def run_tests():
    from zope.testrunner.runner import Runner

    runner = Runner(found_suites=[test_suite()])
    runner.run()
    sys.exit(int(runner.failed))


if __name__ == '__main__':
    run_tests()



from cone.app import cfg
from cone.app import get_root
from cone.app import main_hook
from cone.app import register_config
from cone.app.ugm import ugm_backend
from cone.app.ugm import UGMFactory
from cone.ldap import browser
from cone.ldap.browser import static_resources
from cone.ldap.settings import ldap_cfg
from cone.ldap.settings import LDAPGroupsSettings
from cone.ldap.settings import LDAPRolesSettings
from cone.ldap.settings import LDAPServerSettings
from cone.ldap.settings import LDAPUsersSettings
from node.ext.ldap.ugm import Ugm
import logging


logger = logging.getLogger('cone.ldap')


@main_hook
def initialize_ldap(config, global_config, settings):
    # Set LDAP related config file paths
    ldap_cfg.server_config = settings.get('ldap.server_config', '')
    ldap_cfg.users_config = settings.get('ldap.users_config', '')
    ldap_cfg.groups_config = settings.get('ldap.groups_config', '')
    ldap_cfg.roles_config = settings.get('ldap.roles_config', '')

    # Register general LDAP server settings
    register_config('ldap_server', LDAPServerSettings)

    # Register user, groud and role related settings only if LDAP used
    # as UGM backend
    if settings.get('ugm.backend') == 'ldap':
        register_config('ldap_users', LDAPUsersSettings)
        register_config('ldap_groups', LDAPGroupsSettings)
        register_config('ldap_roles', LDAPRolesSettings)

        # custom UGM styles
        cfg.merged.css.protected.append((static_resources, 'styles.css'))

    # add translation
    config.add_translation_dirs('cone.ldap:locale/')

    config.scan(browser)


@ugm_backend('ldap')
class LDAPUGMFactory(UGMFactory):
    """UGM backend factory for LDAP based UGM implementation.
    """

    def __init__(self, settings):
        """UGM Settings are written to config object by main hook.
        """

    def __call__(self):
        settings = get_root()['settings']
        server_settings = settings['ldap_server']
        if not server_settings.ldap_connectivity:
            logger.error(
                'Could not initialize authentication implementation. '
                'LDAP Server is not available or invalid credentials.'
            )
            return
        props = server_settings.ldap_props
        users_settings = settings['ldap_users']
        if not users_settings.container_exists:
            logger.error(
                'Could not initialize authentication implementation. '
                'Configured users container invalid.'
            )
            return
        ucfg = users_settings.ldap_ucfg
        groups_settings = settings['ldap_groups']
        gcfg = None
        if groups_settings.container_exists:
            gcfg = groups_settings.ldap_gcfg
        else:
            logger.warning(u"Configured groups container invalid.")
        roles_settings = settings['ldap_roles']
        rcfg = None
        if roles_settings.container_exists:
            rcfg = roles_settings.ldap_rcfg
        else:
            logger.warning(u"Configured roles container invalid.")
        return Ugm(
            name='ldap_ugm',
            props=props,
            ucfg=ucfg,
            gcfg=gcfg,
            rcfg=rcfg
        )



from cone.app import get_root
from cone.app.browser.ajax import AjaxAction
from cone.app.browser.ajax import AjaxMessage
from cone.app.browser.utils import make_url
from cone.ldap import testing
from cone.ldap.browser.settings import CreateContainerAction
from cone.ldap.browser.settings import CreateContainerTrigger
from cone.ldap.browser.settings import ScopeVocabMixin
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from cone.ugm.testing import principals
from pyramid.httpexceptions import HTTPForbidden


class TestBrowserSettings(TileTestCase):
    layer = testing.ldap_layer

    def test_ScopeVocabMixin(self):
        vocab_mixin = ScopeVocabMixin()
        self.assertEqual(vocab_mixin.scope_vocab, [
            ('0', 'BASE'),
            ('1', 'ONELEVEL'),
            ('2', 'SUBTREE')
        ])

    @testing.invalidate_settings
    def test_CreateContainerTrigger(self):
        settings = get_root()['settings']['ldap_users']

        cct = CreateContainerTrigger()
        cct.model = settings
        cct.request = self.layer.new_request()

        self.assertTrue(cct.ldap_connectivity)
        self.assertEqual(
            cct.creation_target,
            'http://example.com/settings/ldap_users'
        )

    @testing.invalidate_settings
    def test_CreateContainerAction(self):
        settings = get_root()['settings']['ldap_users']

        cca = CreateContainerAction()
        cca.model = settings
        cca.request = self.layer.new_request()
        err = self.expectError(
            NotImplementedError,
            lambda: cca.continuation
        )
        expected = (
            'Abstract ``CreateContainerAction`` '
            'does not implement ``continuation``'
        )
        self.assertEqual(str(err), expected)

        class MyCreateContainerAction(CreateContainerAction):
            @property
            def continuation(self):
                url = make_url(self.request, node=self.model)
                return AjaxAction(url, 'content', 'inner', '.ldap_cca')

        cca = MyCreateContainerAction()

        request = self.layer.new_request()
        settings.attrs.users_dn = 'ou=cca,dc=other-domain,dc=com'
        self.assertEqual(cca(settings, request), '')
        self.assertFalse(settings.container_exists)

        self.assertEqual(len(request.environ['cone.app.continuation']), 1)
        message = request.environ['cone.app.continuation'][0]
        self.assertTrue(isinstance(message, AjaxMessage))
        self.assertEqual(message.flavor, 'error')
        self.assertEqual(
            message.payload,
            "Cannot create container: Parent not found. Can't continue."
        )

        request = self.layer.new_request()
        settings.attrs.users_dn = 'ou=cca,dc=my-domain,dc=com'
        self.assertEqual(cca(settings, request), '')
        # need to set users_dn again here because settings get invalidated
        # after container creation
        settings.attrs.users_dn = 'ou=cca,dc=my-domain,dc=com'
        self.assertTrue(settings.container_exists)

        self.assertEqual(len(request.environ['cone.app.continuation']), 2)
        message = request.environ['cone.app.continuation'][0]
        self.assertTrue(isinstance(message, AjaxMessage))
        self.assertEqual(message.flavor, 'info')
        self.assertEqual(message.payload, 'created_principal_container')
        action = request.environ['cone.app.continuation'][1]
        self.assertTrue(isinstance(action, AjaxAction))
        self.assertEqual(action.selector, '.ldap_cca')

    @principals(
        users={
            'editor': {},
            'manager': {}
        },
        roles={
            'editor': ['editor'],
            'manager': ['manager']
        })
    def test_server_settings_tiles(self):
        root = get_root()
        server_settings = root['settings']['ldap_server']
        request = self.layer.new_request()
        # Unauthenticated content tile raises error
        self.expectError(
            HTTPForbidden,
            render_tile,
            server_settings,
            request,
            'content'
        )
        # Form tile raise if not manager
        with self.layer.authenticated('editor'):
            self.expectError(
                HTTPForbidden,
                render_tile,
                server_settings,
                request,
                'editform'
            )
        # Authenticate and render tile
        with self.layer.authenticated('manager'):
            res = render_tile(server_settings, request, 'editform')
        expected = 'form action="http://example.com/settings/ldap_server/edit"'
        self.assertTrue(res.find(expected) > -1)

    @principals(
        users={
            'editor': {},
            'manager': {}
        },
        roles={
            'editor': ['editor'],
            'manager': ['manager']
        })
    def test_users_settings_tiles(self):
        root = get_root()
        users_settings = root['settings']['ldap_users']
        request = self.layer.new_request()
        # Unauthenticated content tile raises error
        self.expectError(
            HTTPForbidden,
            render_tile,
            users_settings,
            request,
            'content'
        )
        # Form tile raise if not manager
        with self.layer.authenticated('editor'):
            self.expectError(
                HTTPForbidden,
                render_tile,
                users_settings,
                request,
                'editform'
            )
        # Authenticate and render tile
        with self.layer.authenticated('manager'):
            res = render_tile(users_settings, request, 'editform')
        expected = 'form action="http://example.com/settings/ldap_users/edit"'
        self.assertTrue(res.find(expected) > -1)

    @principals(
        users={
            'editor': {},
            'manager': {}
        },
        roles={
            'editor': ['editor'],
            'manager': ['manager']
        })
    def test_groups_settings_tiles(self):
        root = get_root()
        groups_settings = root['settings']['ldap_groups']
        request = self.layer.new_request()
        # Unauthenticated content tile raises error
        self.expectError(
            HTTPForbidden,
            render_tile,
            groups_settings,
            request,
            'content'
        )
        # Form tile raise if not manager
        with self.layer.authenticated('editor'):
            self.expectError(
                HTTPForbidden,
                render_tile,
                groups_settings,
                request,
                'editform'
            )
        # Authenticate and render tile
        with self.layer.authenticated('manager'):
            res = render_tile(groups_settings, request, 'editform')
        expected = 'form action="http://example.com/settings/ldap_groups/edit"'
        self.assertTrue(res.find(expected) > -1)

    @principals(
        users={
            'editor': {},
            'manager': {}
        },
        roles={
            'editor': ['editor'],
            'manager': ['manager']
        })
    def test_roles_settings_tiles(self):
        root = get_root()
        roles_settings = root['settings']['ldap_roles']
        request = self.layer.new_request()
        # Unauthenticated content tile raises error
        self.expectError(
            HTTPForbidden,
            render_tile,
            roles_settings,
            request,
            'content'
        )
        # Form tile raise if not manager
        with self.layer.authenticated('editor'):
            self.expectError(
                HTTPForbidden,
                render_tile,
                roles_settings,
                request,
                'editform'
            )
        # Authenticate and render tile
        with self.layer.authenticated('manager'):
            res = render_tile(roles_settings, request, 'editform')
        expected = 'form action="http://example.com/settings/ldap_roles/edit"'
        self.assertTrue(res.find(expected) > -1)



from cone.ugm.browser.principal import default_form_field_factory
from cone.ugm.browser.principal import user_field
from functools import partial
from pyramid.static import static_view


static_resources = static_view('static', use_subpath=True)


ldap_cn_field_factory = user_field('cn', backend='ldap')(
    partial(default_form_field_factory, required=True)
)
ldap_sn_field_factory = user_field('sn', backend='ldap')(
    partial(default_form_field_factory, required=True)
)



from cone.app import get_root
from cone.ldap import testing
from cone.ldap.settings import ldap_cfg
from cone.ldap.settings import LDAPGroupsSettings
from cone.ldap.settings import LDAPRolesSettings
from cone.ldap.settings import LDAPServerSettings
from cone.ldap.settings import LDAPUsersSettings
from node.tests import NodeTestCase


class TestLdap(NodeTestCase):
    layer = testing.ldap_layer

    def test_main_hook(self):
        root = get_root()
        settings = root['settings']

        self.assertTrue(isinstance(settings['ldap_groups'], LDAPGroupsSettings))
        self.assertTrue(isinstance(settings['ldap_roles'], LDAPRolesSettings))
        self.assertTrue(isinstance(settings['ldap_server'], LDAPServerSettings))
        self.assertTrue(isinstance(settings['ldap_users'], LDAPUsersSettings))

        self.assertTrue(ldap_cfg.server_config.endswith('ldap_server.xml') > -1)
        self.assertTrue(ldap_cfg.users_config.endswith('ldap_users.xml') > -1)
        self.assertTrue(ldap_cfg.groups_config.endswith('ldap_groups.xml') > -1)
        self.assertTrue(ldap_cfg.roles_config.endswith('ldap_roles.xml') > -1)



from cone.ldap import testing
from cone.tile.tests import TileTestCase
from cone.ugm.tests import test_browser_actions
from cone.ugm.tests import test_browser_autoincrement
from cone.ugm.tests import test_browser_expires
from cone.ugm.tests import test_browser_group
from cone.ugm.tests import test_browser_groups
# from cone.ugm.tests import test_browser_password
from cone.ugm.tests import test_browser_portrait
from cone.ugm.tests import test_browser_principal
from cone.ugm.tests import test_browser_remote
from cone.ugm.tests import test_browser_roles
from cone.ugm.tests import test_browser_root
from cone.ugm.tests import test_browser_settings
from cone.ugm.tests import test_browser_user
from cone.ugm.tests import test_browser_users
from cone.ugm.tests import test_browser_utils
from cone.ugm.tests import test_layout
from cone.ugm.tests import test_localmanager
from cone.ugm.tests import test_model_group
from cone.ugm.tests import test_model_groups
from cone.ugm.tests import test_model_user
from cone.ugm.tests import test_model_users
from cone.ugm.tests import test_settings
from cone.ugm.tests import test_utils
from node.tests import NodeTestCase
import unittest


class TestLayout(
    unittest.TestCase,
    test_layout.TestLayoutBase
):
    pass


class TestModelLocalmanager(
    NodeTestCase,
    test_localmanager.TestModelLocalmanagerBase
):
    layer = testing.ldap_layer


class TestSettings(
    NodeTestCase,
    test_settings.TestSettingsBase
):
    layer = testing.ldap_layer


class TestUtils(
    unittest.TestCase,
    test_utils.TestUtilsBase
):
    layer = testing.ldap_layer


class TestModelGroup(
    unittest.TestCase,
    test_model_group.TestModelGroupBase
):
    layer = testing.ldap_layer


class TestModelGroups(
    NodeTestCase,
    test_model_groups.TestModelGroupsBase
):
    layer = testing.ldap_layer


class TestModelUser(
    unittest.TestCase,
    test_model_user.TestModelUserBase
):
    layer = testing.ldap_layer


class TestModelUsers(
    NodeTestCase,
    test_model_users.TestModelUsersBase
):
    layer = testing.ldap_layer


class TestBrowserActions(
    TileTestCase,
    test_browser_actions.TestBrowserActionsBase
):
    layer = testing.ldap_layer


class TestBrowserAutoincrement(
    TileTestCase,
    test_browser_autoincrement.TestBrowserAutoincrementBase
):
    layer = testing.ldap_layer


class TestBrowserExpires(
    TileTestCase,
    test_browser_expires.TestBrowserExpiresBase
):
    layer = testing.ldap_layer


class TestBrowserGroup(
    TileTestCase,
    test_browser_group.TestBrowserGroupBase
):
    layer = testing.ldap_layer


class TestBrowserGroups(
    TileTestCase,
    test_browser_groups.TestBrowserGroupsBase
):
    layer = testing.ldap_layer


# class TestBrowserPassword(
#     TileTestCase,
#     test_browser_password.TestBrowserPasswordBase
# ):
#     layer = testing.ldap_layer


class TestBrowserPortrait(
    TileTestCase,
    test_browser_portrait.TestBrowserPortraitBase
):
    layer = testing.ldap_layer


class TestBrowserPrincipal(
    TileTestCase,
    test_browser_principal.TestBrowserPrincipalBase
):
    layer = testing.ldap_layer


class TestBrowserRemote(
    TileTestCase,
    test_browser_remote.TestBrowserRemoteBase
):
    layer = testing.ldap_layer


class TestBrowserRoles(
    TileTestCase,
    test_browser_roles.TestBrowserRolesBase
):
    layer = testing.ldap_layer


class TestBrowserRoot(
    TileTestCase,
    test_browser_root.TestBrowserRootBase
):
    layer = testing.ldap_layer


class TestBrowserSettings(
    TileTestCase,
    test_browser_settings.TestBrowserSettingsBase
):
    layer = testing.ldap_layer


class TestBrowserUser(
    TileTestCase,
    test_browser_user.TestBrowserUserBase
):
    layer = testing.ldap_layer


class TestBrowserUsers(
    TileTestCase,
    test_browser_users.TestBrowserUsersBase
):
    layer = testing.ldap_layer


class TestBrowserUtils(
    TileTestCase,
    test_browser_utils.TestBrowserUtilsBase
):
    layer = testing.ldap_layer



from cone.app import get_root
from cone.app.ugm import ugm_backend
from cone.ldap.settings import ldap_cfg
from cone.ugm import testing
from cone.ugm.settings import ugm_cfg
from node.ext.ldap.testing import LDIF_base
from node.ext.ldap.ugm.defaults import creation_defaults
from plone.testing import Layer
import os


base_path = os.path.split(__file__)[0]
ldap_server_config = os.path.join(base_path, 'ldap_server.xml')
ldap_users_config = os.path.join(base_path, 'ldap_users.xml')
ldap_groups_config = os.path.join(base_path, 'ldap_groups.xml')
ldap_roles_config = os.path.join(base_path, 'ldap_roles.xml')


def _invalidate_settings():
    settings = get_root()['settings']
    settings['ugm_general'].invalidate()
    settings['ldap_server'].invalidate()
    settings['ldap_users'].invalidate()
    settings['ldap_groups'].invalidate()
    settings['ldap_roles'].invalidate()


def invalidate_settings(fn):
    """Decorator for tests working on settings nodes.
    """
    def wrapper(*a, **kw):
        _invalidate_settings()
        try:
            fn(*a, **kw)
        finally:
            _invalidate_settings()
    return wrapper


def custom_config_path(fn):
    """Decorator for tests writing to config files.
    """
    def wrapper(*a, **kw):
        ugm_settings = ugm_cfg.ugm_settings
        server_config = ldap_cfg.server_config
        users_config = ldap_cfg.users_config
        groups_config = ldap_cfg.groups_config
        roles_config = ldap_cfg.roles_config
        _invalidate_settings()
        try:
            fn(*a, **kw)
        finally:
            ugm_cfg.ugm_settings = ugm_settings
            ldap_cfg.server_config = server_config
            ldap_cfg.users_config = users_config
            ldap_cfg.groups_config = groups_config
            ldap_cfg.roles_config = roles_config
            _invalidate_settings()
    return wrapper


def rdn_value(node, uid):
    return uid.split('=')[1]


def create_mail(node, uid):
    return '%s@example.com' % rdn_value(node, uid)


creation_defaults['inetOrgPerson'] = dict()
creation_defaults['inetOrgPerson']['sn'] = rdn_value
creation_defaults['inetOrgPerson']['cn'] = rdn_value
creation_defaults['inetOrgPerson']['mail'] = create_mail


class LDAPLayer(testing.UGMLayer, Layer):
    defaultBases = (LDIF_base,)

    def __init__(self):
        Layer.__init__(self)

    def make_app(self):
        super(testing.UGMLayer, self).make_app(**{
            'cone.plugins': '\n'.join([
                'cone.ugm',
                'cone.ldap'
            ]),
            'ugm.backend': 'ldap',
            'ugm.config': testing.ugm_config,
            'ugm.localmanager_config': testing.localmanager_config,
            'ldap.server_config': ldap_server_config,
            'ldap.users_config': ldap_users_config,
            'ldap.groups_config': ldap_groups_config,
            'ldap.roles_config': ldap_roles_config
        })
        settings = get_root()['settings']
        settings['ldap_users'].create_container()
        settings['ldap_groups'].create_container()
        settings['ldap_roles'].create_container()
        ugm_backend.initialize()


ldap_layer = LDAPLayer()
